@@ -92,6 +92,22 @@ net_device_wifi_get_title_widget (NetDeviceWifi *device_wifi)
         return GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder, "center_box"));
 }
 
+void
+net_device_wifi_request_scan (NetDeviceWifi *device_wifi)
+{
+        NMDevice *nm_device;
+        GError *error = NULL;
+
+        nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
+        g_return_if_fail (nm_device_get_device_type (nm_device) == NM_DEVICE_TYPE_WIFI);
+
+        nm_device_wifi_request_scan (NM_DEVICE_WIFI (nm_device), NULL, &error);
+        if (error) {
+                g_debug ("scan request failed: %s", error->message);
+                g_error_free (error);
+        }
+}
+
 static GtkWidget *
 device_wifi_proxy_add_to_stack (NetObject    *object,
                                 GtkStack     *stack,
@@ -252,13 +268,59 @@ get_ap_security_string (NMAccessPoint *ap)
 }
 
 static void
-net_device_wifi_access_point_changed (NMDeviceWifi *nm_device_wifi,
+nm_access_point_property_changed (NMAccessPoint *ap,
+                                  GParamSpec    *pspec,
+                                  NetDeviceWifi *device_wifi)
+{
+        nm_device_wifi_refresh_ui (device_wifi);
+}
+
+static void
+nm_device_wifi_connect_access_points (NMDeviceWifi *nm_device_wifi,
+                                      NetDeviceWifi *net_device_wifi)
+{
+        guint i;
+        const GPtrArray *aps;
+        NMAccessPoint *ap;
+
+        aps = nm_device_wifi_get_access_points (nm_device_wifi);
+        if (!aps)
+                return;
+
+        for (i = 0; i < aps->len; i++) {
+                ap = NM_ACCESS_POINT (g_ptr_array_index (aps, i));
+
+                /* avoid redundant signal handlers */
+                if (g_signal_handler_find (ap, G_SIGNAL_MATCH_FUNC,
+                                           0, 0, NULL,
+                                           &nm_access_point_property_changed,
+                                           NULL) != 0)
+                        continue;
+
+                g_signal_connect_object (ap, "notify",
+                                         G_CALLBACK (nm_access_point_property_changed),
+                                         net_device_wifi, 0);
+        }
+
+}
+
+static void
+net_device_wifi_access_point_added (NMDeviceWifi *nm_device_wifi,
+                                    NMAccessPoint *ap,
+                                    gpointer user_data)
+{
+        NetDeviceWifi *device_wifi = NET_DEVICE_WIFI (user_data);
+
+        populate_ap_list (device_wifi);
+        nm_device_wifi_connect_access_points (nm_device_wifi, device_wifi);
+}
+
+static void
+net_device_wifi_access_point_removed (NMDeviceWifi *nm_device_wifi,
                                       NMAccessPoint *ap,
                                       gpointer user_data)
 {
-        NetDeviceWifi *device_wifi;
-
-        device_wifi = NET_DEVICE_WIFI (user_data);
+        NetDeviceWifi *device_wifi = NET_DEVICE_WIFI (user_data);
 
         populate_ap_list (device_wifi);
 }
@@ -418,9 +480,10 @@ device_get_hotspot_security_details (NetDeviceWifi *device_wifi,
          * We'll refresh the UI when secrets arrive.
          */
         if (tmp_secret == NULL) {
+                GCancellable *cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
                 nm_remote_connection_get_secrets_async ((NMRemoteConnection*)c,
                                                         NM_SETTING_WIRELESS_SECURITY_SETTING_NAME,
-                                                        NULL,
+                                                        cancellable,
                                                         get_secrets_cb,
                                                         device_wifi);
                 return;
@@ -737,6 +800,7 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
         NMDevice *device;
         NMSettingWireless *setting_wireless;
         NMClient *client;
+        GCancellable *cancellable;
 
         if (device_wifi->priv->updating_device)
                 goto out;
@@ -777,12 +841,13 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
 
         /* activate the connection */
         client = net_object_get_client (NET_OBJECT (device_wifi));
+        cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
         if (connection_activate != NULL) {
                 nm_client_activate_connection_async (client,
                                                      connection_activate,
                                                      device,
                                                      NULL,
-                                                     NULL,
+                                                     cancellable,
                                                      connection_activate_cb,
                                                      device_wifi);
                 goto out;
@@ -817,7 +882,7 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
                                                              partial,
                                                              device,
                                                              ap_object_path,
-                                                             NULL,
+                                                             cancellable,
                                                              connection_add_activate_cb,
                                                              device_wifi);
                 if (!allowed_to_share)
@@ -1071,6 +1136,7 @@ overwrite_ssid_cb (GObject      *source_object,
         NMDevice *device;
         NMConnection *c;
         NetDeviceWifi *device_wifi;
+        GCancellable *cancellable;
 
         connection = NM_REMOTE_CONNECTION (source_object);
 
@@ -1085,6 +1151,7 @@ overwrite_ssid_cb (GObject      *source_object,
         device_wifi = user_data;
         device = net_device_get_nm_device (NET_DEVICE (device_wifi));
         client = net_object_get_client (NET_OBJECT (device_wifi));
+        cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
         c = net_device_wifi_get_hotspot_connection (device_wifi);
 
         g_debug ("activate existing hotspot connection\n");
@@ -1092,7 +1159,7 @@ overwrite_ssid_cb (GObject      *source_object,
                                              c,
                                              device,
                                              NULL,
-                                             NULL,
+                                             cancellable,
                                              activate_cb,
                                              device_wifi);
 }
@@ -1112,6 +1179,7 @@ start_shared_connection (NetDeviceWifi *device_wifi)
         NMClient *client;
         const char *mode;
         NMDeviceWifiCapabilities caps;
+        GCancellable *cancellable;
 
         device = net_device_get_nm_device (NET_DEVICE (device_wifi));
         g_assert (nm_device_get_device_type (device) == NM_DEVICE_TYPE_WIFI);
@@ -1121,6 +1189,7 @@ start_shared_connection (NetDeviceWifi *device_wifi)
         ssid = generate_ssid_for_hotspot (device_wifi);
 
         client = net_object_get_client (NET_OBJECT (device_wifi));
+        cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
         if (c != NULL) {
                 NMSettingWireless *sw;
                 const char *c_path;
@@ -1137,7 +1206,7 @@ start_shared_connection (NetDeviceWifi *device_wifi)
 
                 nm_remote_connection_commit_changes_async (connection,
                                                            TRUE,
-                                                           NULL,
+                                                           cancellable,
                                                            overwrite_ssid_cb,
                                                            device_wifi);
                 return;
@@ -1215,7 +1284,7 @@ start_shared_connection (NetDeviceWifi *device_wifi)
                                                      c,
                                                      device,
                                                      NULL,
-                                                     NULL,
+                                                     cancellable,
                                                      activate_new_cb,
                                                      device_wifi);
 
@@ -1480,12 +1549,15 @@ net_device_wifi_constructed (GObject *object)
         nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
 
         g_signal_connect_object (nm_device, "access-point-added",
-                                 G_CALLBACK (net_device_wifi_access_point_changed),
+                                 G_CALLBACK (net_device_wifi_access_point_added),
                                  device_wifi, 0);
         g_signal_connect_object (nm_device, "access-point-removed",
-                                 G_CALLBACK (net_device_wifi_access_point_changed),
+                                 G_CALLBACK (net_device_wifi_access_point_removed),
                                  device_wifi, 0);
 
+        nm_device_wifi_connect_access_points (NM_DEVICE_WIFI (nm_device),
+                                              device_wifi);
+
         /* only enable the button if the user can create a hotspot */
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "start_hotspot_button"));
@@ -1596,6 +1668,7 @@ really_forget (GtkDialog *dialog, gint response, gpointer data)
         GList *r;
         NMRemoteConnection *connection;
         NetDeviceWifi *device_wifi;
+        GCancellable *cancellable;
 
         gtk_widget_destroy (GTK_WIDGET (dialog));
 
@@ -1603,12 +1676,12 @@ really_forget (GtkDialog *dialog, gint response, gpointer data)
                 return;
 
         device_wifi = NET_DEVICE_WIFI (g_object_get_data (G_OBJECT (forget), "net"));
+        cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
         rows = g_object_steal_data (G_OBJECT (forget), "rows");
         for (r = rows; r; r = r->next) {
                 row = r->data;
                 connection = g_object_get_data (G_OBJECT (row), "connection");
-                //FIXME cancellable
-                nm_remote_connection_delete_async (connection, NULL, really_forgotten, device_wifi);
+                nm_remote_connection_delete_async (connection, cancellable, really_forgotten, device_wifi);
                 gtk_widget_destroy (row);
         }
         g_list_free (rows);
@@ -2125,6 +2198,7 @@ ap_activated (GtkListBox *list, GtkListBoxRow *row, NetDeviceWifi *device_wifi)
         NMDevice *nm_device;
         GtkWidget *edit;
         GtkWidget *stack;
+        GCancellable *cancellable;
 
         connection = NM_CONNECTION (g_object_get_data (G_OBJECT (row), "connection"));
         ap = NM_ACCESS_POINT (g_object_get_data (G_OBJECT (row), "ap"));
@@ -2137,9 +2211,10 @@ ap_activated (GtkListBox *list, GtkListBoxRow *row, NetDeviceWifi *device_wifi)
                         gtk_widget_hide (edit);
                         client = net_object_get_client (NET_OBJECT (device_wifi));
                         nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
+                        cancellable = net_object_get_cancellable (NET_OBJECT (device_wifi));
                         nm_client_activate_connection_async (client,
                                                              connection,
-                                                             nm_device, NULL, NULL,
+                                                             nm_device, NULL, cancellable,
                                                              connection_activate_cb, device_wifi);
                 } else {
                         GBytes *ssid;
