@@ -102,6 +102,7 @@ struct _CcPrintersPanelPrivate
   gboolean  new_printer_on_network;
 
   gchar    *renamed_printer_name;
+  gchar    *old_printer_name;
   gchar    *deleted_printer_name;
 
   GHashTable *printer_entries;
@@ -213,6 +214,7 @@ cc_printers_panel_dispose (GObject *object)
   g_clear_pointer (&priv->new_printer_make_and_model, g_free);
 
   g_clear_pointer (&priv->renamed_printer_name, g_free);
+  g_clear_pointer (&priv->old_printer_name, g_free);
 
   if (priv->builder)
     {
@@ -731,6 +733,23 @@ on_printer_deleted (PpPrinterEntry *printer_entry,
   priv->remove_printer_timeout_id = g_timeout_add_seconds (10, on_remove_printer_timeout, self);
 }
 
+static void
+on_printer_renamed (PpPrinterEntry *printer_entry,
+                    gchar          *new_name,
+                    gpointer        user_data)
+{
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  CcPrintersPanelPrivate *priv;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  g_object_get (printer_entry,
+                "printer-name",
+                &priv->old_printer_name,
+                NULL);
+  priv->renamed_printer_name = g_strdup (new_name);
+}
+
 static void
 on_printer_changed (PpPrinterEntry *printer_entry,
                     gpointer        user_data)
@@ -766,6 +785,10 @@ add_printer_entry (CcPrintersPanel *self,
                     "printer-delete",
                     G_CALLBACK (on_printer_deleted),
                     self);
+  g_signal_connect (printer_entry,
+                    "printer-renamed",
+                    G_CALLBACK (on_printer_renamed),
+                    self);
 
   gtk_list_box_insert (GTK_LIST_BOX (content), GTK_WIDGET (printer_entry), -1);
   gtk_widget_show_all (content);
@@ -778,17 +801,24 @@ set_current_page (GObject      *source_object,
                   GAsyncResult *result,
                   gpointer      user_data)
 {
-  GtkWidget *widget = GTK_WIDGET (user_data);
-  PpCups    *cups = PP_CUPS (source_object);
-  gboolean   success;
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel *) user_data;
+  GtkWidget              *widget;
+  PpCups                 *cups = PP_CUPS (source_object);
+  gboolean               success;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
 
   success = pp_cups_connection_test_finish (cups, result);
   g_object_unref (source_object);
 
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "main-vbox");
   if (success)
     gtk_stack_set_visible_child_name (GTK_STACK (widget), "empty-state");
   else
     gtk_stack_set_visible_child_name (GTK_STACK (widget), "no-cups-page");
+
+  update_sensitivity (user_data);
 }
 
 static void
@@ -801,6 +831,7 @@ actualize_printers_list_cb (GObject      *source_object,
   GtkWidget              *widget;
   PpCups                 *cups = PP_CUPS (source_object);
   PpCupsDests            *cups_dests;
+  gboolean                new_printer_available = FALSE;
   GError                 *error = NULL;
   int                     i;
 
@@ -826,19 +857,32 @@ actualize_printers_list_cb (GObject      *source_object,
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "main-vbox");
   if (priv->num_dests == 0 && !priv->new_printer_name)
-    pp_cups_connection_test_async (g_object_ref (cups), set_current_page, widget);
+    pp_cups_connection_test_async (g_object_ref (cups), set_current_page, self);
   else
     gtk_stack_set_visible_child_name (GTK_STACK (widget), "printers-list");
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "content");
   gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_destroy, NULL);
+
+  for (i = 0; i < priv->num_dests; i++)
+    {
+      new_printer_available = g_strcmp0 (priv->dests[i].name, priv->renamed_printer_name) == 0;
+      if (new_printer_available)
+        break;
+    }
+
   for (i = 0; i < priv->num_dests; i++)
     {
       if (g_strcmp0 (priv->dests[i].name, priv->deleted_printer_name) == 0)
           continue;
 
+      if (new_printer_available && g_strcmp0 (priv->dests[i].name, priv->old_printer_name) == 0)
+          continue;
+
       add_printer_entry (self, priv->dests[i]);
     }
+
+  update_sensitivity (user_data);
 }
 
 static void
@@ -992,6 +1036,15 @@ update_sensitivity (gpointer user_data)
       cups_server[0] != '/')
     local_server = FALSE;
 
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "headerbar-buttons");
+  gtk_widget_set_visible (widget, !no_cups);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "search-button");
+  gtk_widget_set_visible (widget, !no_cups);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "search-bar");
+  gtk_widget_set_visible (widget, !no_cups);
+
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button");
   gtk_widget_set_sensitive (widget, local_server && priv->is_authorized && !no_cups && !priv->new_printer_name);
 
@@ -1197,6 +1250,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   priv->new_printer_on_network = FALSE;
 
   priv->renamed_printer_name = NULL;
+  priv->old_printer_name = NULL;
   priv->deleted_printer_name = NULL;
 
   priv->permission = NULL;
